#pragma once

/*! The result of CAL_GetObjectType if the object is a box. */
#define CAL_BOX                        1
/*! The result of CAL_GetObjectType if the object is a sphere. */
#define CAL_SPHERE                     2
/*! The result of CAL_GetObjectType if the object is a cylinder. */
#define CAL_CYLINDER                   3
/*! The result of CAL_GetObjectType if the object is a cone. */
#define CAL_CONE                       4
/*! The result of CAL_GetObjectType if the object is a triangle group. */
#define CAL_TRIANGLES                  5
/*! The result of CAL_GetObjectType if the object is a polygon. */
#define CAL_POLYGON                    6
/*! The result of CAL_GetObjectType if the object is a polygon group. */
#define CAL_POLYGONGROUP               7
/*! The result of CAL_GetObjectType if the object is a polyline. */
#define CAL_POLYLINE                   8
/*! The result of CAL_GetObjectType if the object is a tetrahedron. */
#define CAL_TETRAHEDRON                9

/*! Retrieve information about the last collision check/closest pair/penetration depth. \ingroup AdvancedGlobal */
struct SCALResult
{
  /*! The first resulting object. */
  int objID0;
  /*! The second resulting object. */
  int objID1;
  /*! The first point involved in the check. Only applicable in case of closest pair and penetration depth. */
  CAL_scalar vector0[3];
  /*! The second point involved in the check. Only applicable in case of closest pair and penetration depth. */
  CAL_scalar vector1[3];
  /*! The penetration distance or distance between the groups, depending on the type of check. */
  CAL_scalar distance;
};

/*! The structure used with CAL_GetGroup to retrieve group information. \ingroup Retrieval */
struct SCALGroup
{
  /*! The number of direct subgroups this group has. */
  int nrChildren;
  /*! The number of objects this group contains (not including the potential subgroups). */
  int nrObjects;
  /*! The visibility settings for all of the 4 views. */
  bool visible[4];
  /*! Is the group collision capable (is it in the collision checker)? */
  bool colCapable;
};

/*! A structure used with CAL_GetObject to retrieve information about a box. \ingroup Retrieval */
struct SCALBox
{
  /*! The ID of the group the object is in. */
  int groupID;
  /*! The current world matrix of the object. */
  CAL_scalar matrix[4][4];
  /*! The width of the box. */
  CAL_scalar width;
  /*! The height of the box. */
  CAL_scalar height;
  /*! The depth of the box. */
  CAL_scalar depth;
};

/*! A structure used with CAL_GetObject to retrieve information about a sphere. \ingroup Retrieval */
struct SCALSphere
{
  /*! The ID of the group the object is in. */
  int groupID;
  /*! The current world matrix of the object. */
  CAL_scalar matrix[4][4];
  /*! The radius of the sphere. */
  CAL_scalar radius;
};

/*! A structure used with CAL_GetObject to retrieve information about a cylinder. \ingroup Retrieval */
struct SCALCylinder
{
  /*! The ID of the group the object is in. */
  int groupID;
  /*! The current world matrix of the object. */
  CAL_scalar matrix[4][4];
  /*! The radius of the cylinder. */
  CAL_scalar radius;
  /*! The height of the cylinder. */
  CAL_scalar height;
};

/*! A structure used with CAL_GetObject to retrieve information about a cone. \ingroup Retrieval */
struct SCALCone
{
  /*! The ID of the group the object is in. */
  int groupID;
  /*! The current world matrix of the object. */
  CAL_scalar matrix[4][4];
  /*! The radius of the cone. */
  CAL_scalar radius;
  /*! The height of the cone. */
  CAL_scalar height;
};

/*! A structure used with CAL_GetObject to retrieve information about a group of triangles. \ingroup Retrieval */
struct SCALTriangles
{
  /*! The ID of the group the object is in. */
  int groupID;
  /*! The current world matrix of the object. */
  CAL_scalar matrix[4][4];
  /*! The number of triangles in this object. */
  int nrTriangles;
  /*! The points of the triangles, 9 values for every triangle. */
  CAL_scalar *points;
};

/*! A structure used with CAL_GetObject to retrieve information about a polygon. \ingroup Retrieval */
struct SCALPolygon
{
  /*! The ID of the group the object is in. */
  int groupID;
  /*! The current world matrix of the object. */
  CAL_scalar matrix[4][4];
  /*! The number of points the polygon contains. */
  int nrPoints;
  /*! The coordinates of the polygon, 3 values for every point. */
  CAL_scalar *points;
};

/*! A structure used with CAL_GetObject to retrieve information about a polygon group. \ingroup Retrieval */
struct SCALPolygonGroup
{
  /*! The ID of the group the object is in. */
  int groupID;
  /*! The current world matrix of the object. */
  CAL_scalar matrix[4][4];
  /*! The number of polygons. */
  int nrPolygons;
  /*! The number of points per polygon. The length of this list is determined by nrPolygons. */
  int *nrPoints;
  /*! The coordinates of the polygons, 3 values for every point. */
  CAL_scalar *points;
};

/*! A structure used with CAL_GetObject to retrieve information about a poly line. \ingroup Retrieval */
struct SCALPolyline
{
  /*! The ID of the group the object is in. */
  int groupID;
  /*! The current world matrix of the object. */
  CAL_scalar matrix[4][4];
  /*! The number of lines. */
  int nrLines;
  /*! The number of points per line. The length of this list is determined by nrLines. */
  int *nrPoints;
  /*! The coordinates of the lines, 3 values per point. */
  CAL_scalar *points;
};

/*! A structure used with CAL_GetObject to retrieve information about a tetrahedron. \ingroup Retrieval */
struct SCALTetrahedron
{
  /*! The ID of the group the object is in. */
  int groupID;
  /*! The current world matrix of the object. */
  CAL_scalar matrix[4][4];
  /*! The coordinates of the tetrahedron. The length of this list is 3*4=12 values. */
  CAL_scalar *points;
};

/*! \file callistoTypes.h
*/